"""___anubis - Framework for running bpp qa"""
from . import arg_parser_main, parallelizer, results, text
__version__ = '0.1.0'
__author__ = 'matthew bahloul <matthew.bahloul@blueprintprep.com>'
__all__ = ['arg_parser_main', 'text', 'parallelizer', 'results']
